// vpm-local.h
//
// local definitions for vpm simulator
//
// Jun Makino 2004/6/7

#ifndef __VPM_LOCAL__
#define __VPM_LOCAL__

#define MAXVECTORLENGTH 4
// size of regsters in long words
#define BMEMSIZE 1024
#define LMEMSIZE 256
#define GRFSIZE 32
#define TREGSIZE MAXVECTORLENGTH
#define MASKDEPTH 8

#define MAXIDPPACKETLENGTH LMEMSIZE
#define MAXTEXTLENGTH 256

#define LMHISTLEN 4

// number of pes per bb and bbs per chip
#ifndef NPE
#define NPE 2
#endif
#ifndef NBB
#define NBB 2
#endif
#define NPEMAX 64
#define NBBMAX 64

// address map of BM
// 0-7ff real memory
// 800   bb control register
// 1000  rrn command register
#define BBCREGADDR 0x800
#define RRNCMDREGADDR 0x1000

typedef  unsigned long long  UINT64;
typedef  long long           INT64;
typedef  unsigned            UINT;
typedef  int                 INT;
typedef  unsigned            UINT32;
typedef  int                 INT32;

typedef UINT64 SWORD;

typedef struct lword_struct{
    SWORD sw[2];
}LWORD,*PLWORD;

typedef struct fword_struct{
    UINT sign;
    UINT exponent;
    UINT64 mantissa;
}FWORD,*PFWORD;

typedef struct mask_struct{
    int data[MASKDEPTH][TREGSIZE];
}MASKREG,*PMASKREG;

typedef struct rrnpack{
    INT adr, len, bbadr, reduc, wl, fsel,
	fadd_normala, fadd_normalb, fadd_signb,
	fadd_round, fadd_normalo,
	ialu_ialuop, ialu_unsigned, odpoe,sregen;
}RRNPACK, *PRRNPACK;



typedef struct pe_struct{
    int id;
    LWORD treg[TREGSIZE];
    LWORD tregold[2][TREGSIZE];
    LWORD fmuloutreg[TREGSIZE];
    LWORD fmuloutregold[2][TREGSIZE];
    LWORD grf[GRFSIZE];
    LWORD grfold[2][GRFSIZE];
    LWORD lmem[LMEMSIZE];
    LWORD lmemold[2][LMEMSIZE];
    MASKREG mreg;
    LWORD peid;
    LWORD bmid;
    INT l_write_old;
    INT l_write_old2;
    INT r_write_old;
}PE,*PPE;


typedef struct bb_struct{
    int id;
    LWORD bmem[BMEMSIZE];
    PE pearray[NPE];
    LWORD bmid;
    INT globalblockmask;
    INT blockmask;
    INT errorstatus;
}BB,*PBB;

typedef struct chip_struct{
    BB bbarray[NBB];
    INT lmhistory[LMHISTLEN];
    INT status;
    INT activebbs;
}CHIP,*PCHIP;

typedef struct rrnstate_struct{
    RRNPACK code;
    int convtype;
    int status;
    int nread;
    void * data;
}RRNSTATE, *PRRNSTATE;

typedef struct idpstate_struct{
    INT count;
}IDPSTATE, *PIDPSTATE;


#define SW_M_LEN 24
#define SW_E_LEN 11
#define SW_LEN (SW_M_LEN+SW_E_LEN+1)
#define LW_M_LEN 60
#define LW_E_LEN 11
#define LW_LEN (LW_M_LEN+LW_E_LEN+1)
#define D_M_LEN 52
#define U64LEN   64
#define U32LEN   32
#define MUL_AINPUT_LEN 50
#define MUL_BINPUT_LEN 25


static const UINT64 oneu64 = 1;
#define BITMASK(x)  ((oneu64 << x)-1)

static const UINT64 swmmask = (((UINT64)1)<<SW_M_LEN)-1;
static const UINT64 swmpmask = (((UINT64)1)<<(SW_M_LEN+1))-1;
static const UINT64 swhbit = (((UINT64)1)<<SW_M_LEN);
static const UINT64 swemask = (((UINT64)1)<<SW_E_LEN)-1;
static const INT    sweoff = (1<<(SW_E_LEN-1))-1;


typedef union dlpack{
    UINT64        i64;
    double       d;
}DATAPACK;

typedef union ldlpack{
    UINT64        i64[2];
    long double       d;
}LONGDATAPACK;



typedef struct isppack{
    INT ll, m_imr, m_omr, m_ifsel,
	t_write, t_shortstop, t_isel,
	r_write, r_isel, r_wadr, r_wadri, r_wwl,
	r_radra, r_radria, r_rwla, r_radrb, r_radrib, r_rwlb,
	l_write, l_isel, l_adr, l_adri, l_tregadr, l_wl,
	fmul_shift50a, fmul_round50a, fmul_rounda, fmul_normala,
	fmul_shift25b, fmul_shift50b, fmul_roundb, fmul_normalb,
	fmul_round, fmul_normalo, fmul_isela, fmul_iselb, fadd_normala,
	fadd_normalb, fadd_signb, fadd_round, fadd_normalo,
	fadd_isela, fadd_iselb,
	ialu_ialuop, ialu_unsigned, fsel,
	bm_write, bm_adr, bm_peadr, bm_wl;
    int idpmode;
    int idpcode;
    int idpbcastmode;
    int idpbbid;
    int rrnmode;
    RRNPACK rrndata;
    int rrnconvtype;
    int ispflag;
    char instructionstring[MAXTEXTLENGTH];
}ISPPACK,*PISPPACK;



#endif /* __VPM_LOCAL__*/
